@@ -7,6 +7,8 @@ import os
 from tableone import TableOne
 from collections import defaultdict
 from io import StringIO
+from .gene_patterns import *
+import plotly.express as px
 
 import pypeta
 from pypeta import Peta
@@ -202,7 +204,7 @@ class CDx_Data():
         self.cli = self._infer_datetime_columns()
         self.crosstab = self.get_crosstab()
 
-    def write_files(self, path: str = './'):
+    def to_tsvs(self, path: str = './'):
         """Write CDx_Data properties to 4 seprated files
 
         Args:
@@ -225,6 +227,24 @@ class CDx_Data():
                            index=None,
                            sep='\t')
 
+    def to_excel(self, filename: str = './output.xlsx'):
+        """Write CDx_Data properties to excel file
+
+        Args:
+            filename (str, optional): target filename. Defaults to './output.xlsx'.
+        """
+        if not filename.endswith('xlsx'):
+            filename = filename + '.xlsx'
+        with pd.ExcelWriter(filename) as ew:
+            if not self.cli is None:
+                self.cli.to_excel(ew, sheet_name='clinical', index=None)
+            if not self.mut is None:
+                self.mut.to_excel(ew, sheet_name='mutations', index=None)
+            if not self.cnv is None:
+                self.cnv.to_excel(ew, sheet_name='cnv', index=None)
+            if not self.sv is None:
+                self.sv.to_excel(ew, sheet_name='sv', index=None)
+
     def _set_cli(self):
         """Set the cli attribute, generate a void DataFrame when it is not specified. 
         """
@@ -488,23 +508,23 @@ class CDx_Data():
         # multi hit represent as a string
         if fuzzy:
             cli_df['queryId'] = cli_df['sampleId'].map(
-                lambda x: ','.join(fuzzy_to_origin[transform(x)]))
+                lambda x: ','.join(fuzzy_to_origin[transform(x)])).copy()
 
         if not self.mut is None and len(self.mut) != 0:
             mut_df = self.mut[self.mut['Tumor_Sample_Barcode'].isin(
-                cli_df['sampleId'])]
+                cli_df['sampleId'])].copy()
         else:
             mut_df = None
 
         if not self.cnv is None and len(self.cnv) != 0:
             cnv_df = self.cnv[self.cnv['Tumor_Sample_Barcode'].isin(
-                cli_df['sampleId'])]
+                cli_df['sampleId'])].copy()
         else:
             cnv_df = None
 
         if not self.sv is None and len(self.sv) != 0:
             sv_df = self.sv[self.sv['Tumor_Sample_Barcode'].isin(
-                cli_df['sampleId'])]
+                cli_df['sampleId'])].copy()
         else:
             sv_df = None
 
@@ -790,6 +810,64 @@ class CDx_Data():
     def survival(self):
         pass
 
+    def plot_gene_variant_rate(self, genes=genes_688):
+        freq_mut_s = self.test_positive_rate(
+            genes_to_observe=genes,
+            groupby_genes=True,
+            variant_type_to_observe=['MUTATIONS']) * 100
+        freq_cnv_s = self.test_positive_rate(genes_to_observe=genes,
+                                             groupby_genes=True,
+                                             variant_type_to_observe=['CNV'
+                                                                      ]) * 100
+        freq_sv_s = self.test_positive_rate(genes_to_observe=genes,
+                                            groupby_genes=True,
+                                            variant_type_to_observe=['FUSION'
+                                                                     ]) * 100
+
+        #判定是否三种变异类型是0并处理
+        avalible_s = []
+        variantlist = [freq_mut_s, freq_cnv_s, freq_sv_s]
+        for i, x in enumerate(variantlist):
+            if len(x) != 0:
+                avalible_s.append(x)
+
+        if len(avalible_s) == 0:
+            return 'no data'
+
+        if len(freq_mut_s) == 0:
+            freq_mut_s = pd.Series([0] * len(avalible_s[0]),
+                                   index=avalible_s[0].index)
+
+        if len(freq_cnv_s) == 0:
+            freq_cnv_s = pd.Series([0] * len(avalible_s[0]),
+                                   index=avalible_s[0].index)
+
+        if len(freq_sv_s) == 0:
+            freq_sv_s = pd.Series([0] * len(avalible_s[0]),
+                                  index=avalible_s[0].index)
+
+        freq_s = pd.DataFrame({
+            'Mutations': freq_mut_s,
+            'CNV': freq_cnv_s,
+            'SV': freq_sv_s
+        }).fillna(0)
+
+        freq_s['total'] = freq_s.sum(axis=1)
+
+        freq_s = freq_s.sort_values(by='total', ascending=False)
+
+        fig = px.bar(
+            freq_s,
+            x=freq_s.index,
+            y=['Mutations', 'CNV', 'SV'],
+        )
+        #fig.update_traces(texttemplate='%{text:.2%}', textposition='outside',)
+        fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')
+        fig.layout.xaxis.title.text = None
+        fig.layout.yaxis.title.text = '检出率（%）'
+        fig.layout.legend.title.text = None
+        return fig
+
     # 画图的程序是否内置？
     def test_positive_rate(
             self,
